@@ -27,7 +27,7 @@ ax.set_ylim((-1.5, 1.5))
 ax.set_yticks([-1, 0, 1])
 
 # Only draw spine between the y-ticks
-ax.spines['left'].set_bounds(-1, 1)
+ax.spines['left'].set_bounds((-1, 1))
 # Hide the right and top spines
 ax.spines['right'].set_visible(False)
 ax.spines['top'].set_visible(False)
