@@ -44,7 +44,6 @@ In Matplotlib, the location of axes (including subplots) are specified in
 normalized figure coordinates. It can happen that your axis labels or
 titles (or sometimes even ticklabels) go outside the figure area, and are thus
 clipped.
-
 """
 
 # sphinx_gallery_thumbnail_number = 18
@@ -58,19 +57,20 @@ import numpy as np
 
 plt.rcParams['savefig.facecolor'] = "0.8"
 plt.rcParams['figure.figsize'] = 4.5, 4.
+plt.rcParams['figure.max_open_warning'] = 50
 
 
-def example_plot(ax, fontsize=12, nodec=False):
+def example_plot(ax, fontsize=12, hide_labels=False):
     ax.plot([1, 2])
 
     ax.locator_params(nbins=3)
-    if not nodec:
+    if hide_labels:
+        ax.set_xticklabels([])
+        ax.set_yticklabels([])
+    else:
         ax.set_xlabel('x-label', fontsize=fontsize)
         ax.set_ylabel('y-label', fontsize=fontsize)
         ax.set_title('Title', fontsize=fontsize)
-    else:
-        ax.set_xticklabels('')
-        ax.set_yticklabels('')
 
 
 fig, ax = plt.subplots(constrained_layout=False)
@@ -105,8 +105,8 @@ for ax in axs.flat:
 # Colorbars
 # =========
 #
-# If you create a colorbar with the :func:`~matplotlib.pyplot.colorbar`
-# command you need to make room for it.  ``constrained_layout`` does this
+# If you create a colorbar with `.Figure.colorbar`,
+# you need to make room for it.  ``constrained_layout`` does this
 # automatically.  Note that if you specify ``use_gridspec=True`` it will be
 # ignored because this option is made for improving the layout via
 # ``tight_layout``.
@@ -266,19 +266,13 @@ fig.savefig('CL02.png', bbox_inches='tight', dpi=100)
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
 for ax in axs.flat:
-    example_plot(ax, nodec=True)
-    ax.set_xticklabels('')
-    ax.set_yticklabels('')
-fig.set_constrained_layout_pads(w_pad=4./72., h_pad=4./72.,
-        hspace=0., wspace=0.)
+    example_plot(ax, hide_labels=True)
+fig.set_constrained_layout_pads(w_pad=4/72, h_pad=4/72, hspace=0, wspace=0)
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
 for ax in axs.flat:
-    example_plot(ax, nodec=True)
-    ax.set_xticklabels('')
-    ax.set_yticklabels('')
-fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
-        hspace=0., wspace=0.)
+    example_plot(ax, hide_labels=True)
+fig.set_constrained_layout_pads(w_pad=2/72, h_pad=2/72, hspace=0, wspace=0)
 
 ##########################################
 # Spacing between subplots is set by ``wspace`` and ``hspace``. There are
@@ -289,11 +283,8 @@ fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
 for ax in axs.flat:
-    example_plot(ax, nodec=True)
-    ax.set_xticklabels('')
-    ax.set_yticklabels('')
-fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
-        hspace=0.2, wspace=0.2)
+    example_plot(ax, hide_labels=True)
+fig.set_constrained_layout_pads(w_pad=2/72, h_pad=2/72, hspace=0.2, wspace=0.2)
 
 
 ##########################################
@@ -313,8 +304,7 @@ for ax in axs.flat:
     fig.colorbar(pc, ax=ax, shrink=0.6, pad=0)
     ax.set_xticklabels('')
     ax.set_yticklabels('')
-fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
-        hspace=0.2, wspace=0.2)
+fig.set_constrained_layout_pads(w_pad=2/72, h_pad=2/72, hspace=0.2, wspace=0.2)
 
 ##########################################
 # In the above example, the colorbar will not ever be closer than 2 pts to
@@ -327,8 +317,7 @@ for ax in axs.flat:
     fig.colorbar(im, ax=ax, shrink=0.6, pad=0.05)
     ax.set_xticklabels('')
     ax.set_yticklabels('')
-fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
-        hspace=0.2, wspace=0.2)
+fig.set_constrained_layout_pads(w_pad=2/72, h_pad=2/72, hspace=0.2, wspace=0.2)
 
 ##########################################
 # rcParams
@@ -498,17 +487,16 @@ ax2 = fig.add_axes(bb_ax2)
 #
 #
 # Limitations
-# ========================
+# ===========
 #
 # Incompatible functions
 # ----------------------
 #
-# ``constrained_layout`` will not work on subplots
-# created via the `.pyplot.subplot` command.  The reason is that each of these
-# commands creates a separate `.GridSpec` instance and ``constrained_layout``
-# uses (nested) gridspecs to carry out the layout.  So the following fails
-# to yield a nice layout:
-
+# ``constrained_layout`` will not work on subplots created via
+# `.pyplot.subplot`.  The reason is that each call to `.pyplot.subplot` creates
+# a separate `.GridSpec` instance and ``constrained_layout`` uses (nested)
+# gridspecs to carry out the layout.  So the following fails to yield a nice
+# layout:
 
 fig = plt.figure()
 
@@ -789,8 +777,7 @@ plot_children(fig, fig._layoutbox, printit=False)
 
 fig = plt.figure(constrained_layout=True)
 gs = gridspec.GridSpec(3, 2, figure=fig,
-    height_ratios=[1., 0.5, 1.5],
-    width_ratios=[1.2, 0.8])
+                       height_ratios=[1., 0.5, 1.5], width_ratios=[1.2, 0.8])
 ax = fig.add_subplot(gs[:2, 0])
 im = ax.pcolormesh(arr, **pc_kwargs)
 ax = fig.add_subplot(gs[2, 0])
